==> trend_etl-1.4.7.tar.gz: trend_etl-1.4.7/trend_etl/tasks/load_docs.py <==
"""
Load docs table for new documents fetched from facebook, reddit, discord, etc.

NOTE: All load_docs_* functions are safe to re-run (idempotent).
"""

import argparse
from typing import cast

from delta.tables import DeltaTable
from pyspark.sql import SparkSession, DataFrame
from pyspark.sql.streaming import StreamingQuery

from trend_etl.functions.index import (
    full_table_name,
    docs_upsert_submissions,
    docs_upsert_replies,
    docs_upsert_discord,
    get_silver_table,
)
from trend_etl.transform.submissions import from_bronze_get_submissions_df
from trend_etl.transform.replies import from_bronze_get_replies_df
from trend_etl.transform.messages import from_bronze_get_messages_df
import trend_etl.schema.args as arguments
from trend_etl.schema.types import Merge_Strategy

PROGRAM = "load_docs"


def load_docs_submissions(
    spark: SparkSession,
    bronze_catalog: str,
    docs_table: DeltaTable,
    subreddit_idents: DataFrame,
    submission_idents: DataFrame,
    bots: DataFrame,
    strategy: Merge_Strategy,
    checkpoints_path_prefix: str,
) -> StreamingQuery:
    """
    Merge in new documents added to the submissions bronze table.
    """
    bronze_table = full_table_name(
        bronze_catalog,
        arguments.schemas[arguments.schemas_bronze],
        arguments.tables[arguments.tables_submissions],
    )
    streaming = spark.readStream.format("delta").table(bronze_table)
    checkpoint_loc = (
        checkpoints_path_prefix
        + "/docs_noparts/"
        + arguments.tables[arguments.tables_submissions]
    )

    return (
        from_bronze_get_submissions_df(
            streaming, subreddit_idents, submission_idents, bots
        )
        .writeStream.format("delta")
        .foreachBatch(
            lambda batch_df, batch_id: docs_upsert_submissions(
                docs_table, strategy, batch_df
            )
        )
        .option("checkpointLocation", checkpoint_loc)
        .outputMode("append")
        .trigger(availableNow=True)
        .start()
    )


def load_docs_replies(
    spark: SparkSession,
    bronze_catalog: str,
    docs_table: DeltaTable,
    subreddit_idents: DataFrame,
    submission_idents: DataFrame,
    reply_idents: DataFrame,
    bots: DataFrame,
    strategy: Merge_Strategy,
    checkpoints_path_prefix: str,
) -> StreamingQuery:
    """
    Merge in new documents added to the replies bronze table.
    """
    bronze_table = full_table_name(
        bronze_catalog,
        arguments.schemas[arguments.schemas_bronze],
        arguments.tables[arguments.tables_replies],
    )
    streaming = spark.readStream.format("delta").table(bronze_table)
    checkpoint_loc = (
        checkpoints_path_prefix
        + "/docs_noparts/"
        + arguments.tables[arguments.tables_replies]
    )

    return (
        from_bronze_get_replies_df(
            streaming,
            subreddit_idents,
            submission_idents,
            reply_idents,
            bots,
        )
        .writeStream.format("delta")
        .foreachBatch(
            lambda batch_df, batch_id: docs_upsert_replies(
                docs_table, strategy, batch_df
            )
        )
        .option("checkpointLocation", checkpoint_loc)
        .outputMode("append")
        .trigger(availableNow=True)
        .start()
    )


def load_docs_discord(
    spark: SparkSession,
    bronze_catalog: str,
    docs_table: DeltaTable,
    guild_idents: DataFrame,
    channel_idents: DataFrame,
    message_idents: DataFrame,
    strategy: Merge_Strategy,
    checkpoints_path_prefix: str,
) -> StreamingQuery:
    """
    Merge in new documents added to the replies bronze table.
    """
    bronze_table = full_table_name(
        bronze_catalog,
        arguments.schemas[arguments.schemas_bronze],
        arguments.tables[arguments.tables_messages],
    )
    streaming = spark.readStream.format("delta").table(bronze_table)
    checkpoint_loc = (
        checkpoints_path_prefix
        + "/docs_noparts/"
        + arguments.tables[arguments.tables_messages]
    )

    return (
        from_bronze_get_messages_df(
            streaming,
            guild_idents,
            channel_idents,
            message_idents,
        )
        .writeStream.format("delta")
        .foreachBatch(
            lambda batch_df, batch_id: docs_upsert_discord(
                docs_table, strategy, batch_df
            )
        )
        .option("checkpointLocation", checkpoint_loc)
        .outputMode("append")
        .trigger(availableNow=True)
        .start()
    )


def main() -> None:
    """
    Entry point for load docs task
    """
    parser = argparse.ArgumentParser(description=f"Arguments for Trend ETL/{PROGRAM}.")

    parser.add_argument(
        "--bronze-catalog",
        dest="bronze_catalog",
        required=True,
        choices=arguments.catalogs,
        help="The unit catalog to use",
    )

    parser.add_argument(
        "--target-catalog",
        dest="target_catalog",
        required=True,
        choices=arguments.catalogs,
        help="The unit catalog to use",
    )

    parser.add_argument(
        "--checkpoints-path-prefix",
        dest="checkpoints_path_prefix",
        required=True,
        help="The path to the folder (in S3) to store the checkpoints for this table",
    )

    parser.add_argument(
        "--keep-oldest",
        dest="keep_oldest",
        required=True,
        choices=arguments.yesno,
        help="Keep newest or oldest when using the merge to drop duplicates",
    )

    args = parser.parse_args()

    spark = cast(SparkSession, SparkSession.builder.appName(PROGRAM).getOrCreate())

    subreddit_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_subreddit_idents],
    )
    subreddit_idents = spark.read.format("delta").table(subreddit_idents_table)

    submission_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_submission_idents],
    )
    submission_idents = spark.read.format("delta").table(submission_idents_table)

    reply_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_reply_idents],
    )
    reply_idents = spark.read.format("delta").table(reply_idents_table)

    bots_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_reddit_bot_users],
    )
    bots = spark.read.format("delta").table(bots_table)

    guild_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_guild_idents],
    )
    guild_idents = spark.read.format("delta").table(guild_idents_table)

    channel_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_channel_idents],
    )
    channel_idents = spark.read.format("delta").table(channel_idents_table)

    message_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_message_idents],
    )
    message_idents = spark.read.format("delta").table(message_idents_table)

    tablename = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        get_silver_table(args.target_catalog, arguments.tables_docs),
    )

    docs_table = DeltaTable.forName(spark, tablename)
    strategy: Merge_Strategy = (
        "keep-oldest" if args.keep_oldest == "yes" else "keep-newest"
    )

    load_docs_submissions(
        spark,
        args.bronze_catalog,
        docs_table,
        subreddit_idents,
        submission_idents,
        bots,
        strategy,
        args.checkpoints_path_prefix,
    ).awaitTermination()

    load_docs_replies(
        spark,
        args.bronze_catalog,
        docs_table,
        subreddit_idents,
        submission_idents,
        reply_idents,
        bots,
        strategy,
        args.checkpoints_path_prefix,
    ).awaitTermination()

    load_docs_discord(
        spark,
        args.bronze_catalog,
        docs_table,
        guild_idents,
        channel_idents,
        message_idents,
        strategy,
        args.checkpoints_path_prefix,
    ).awaitTermination()


if __name__ == "__main__":
    main()

==> trend_etl-2.0.0a2.tar.gz: trend_etl-2.0.0a2/trend_etl/tasks/load_docs.py <==
"""
Load docs table for new documents fetched from facebook, reddit, discord, etc.

NOTE: All load_docs_* functions are safe to re-run (idempotent).
"""

import argparse
from typing import cast

from delta.tables import DeltaTable
from pyspark.sql import SparkSession, DataFrame
from pyspark.sql.streaming import StreamingQuery

from trend_etl.functions.delta import (
    full_table_name,
    docs_upsert_submissions,
    docs_upsert_replies,
    docs_upsert_discord,
    get_silver_table,
)
from trend_etl.transform.submissions import from_bronze_get_submissions_df
from trend_etl.transform.replies import from_bronze_get_replies_df
from trend_etl.transform.messages import from_bronze_get_messages_df
import trend_etl.schema.args as arguments
from trend_etl.schema.types import Merge_Strategy

PROGRAM = "load_docs"


def load_docs_submissions(
    spark: SparkSession,
    bronze_catalog: str,
    docs_table: DeltaTable,
    subreddit_idents: DataFrame,
    submission_idents: DataFrame,
    bots: DataFrame,
    strategy: Merge_Strategy,
    checkpoints_path_prefix: str,
) -> StreamingQuery:
    """
    Merge in new documents added to the submissions bronze table.
    """
    bronze_table = full_table_name(
        bronze_catalog,
        arguments.schemas[arguments.schemas_bronze],
        arguments.tables[arguments.tables_submissions],
    )
    streaming = spark.readStream.format("delta").table(bronze_table)
    checkpoint_loc = (
        checkpoints_path_prefix
        + "/docs_noparts/"
        + arguments.tables[arguments.tables_submissions]
    )

    return (
        from_bronze_get_submissions_df(
            streaming, subreddit_idents, submission_idents, bots
        )
        .writeStream.format("delta")
        .foreachBatch(
            lambda batch_df, batch_id: docs_upsert_submissions(
                docs_table, strategy, batch_df
            )
        )
        .option("checkpointLocation", checkpoint_loc)
        .outputMode("append")
        .trigger(availableNow=True)
        .start()
    )


def load_docs_replies(
    spark: SparkSession,
    bronze_catalog: str,
    docs_table: DeltaTable,
    subreddit_idents: DataFrame,
    submission_idents: DataFrame,
    reply_idents: DataFrame,
    bots: DataFrame,
    strategy: Merge_Strategy,
    checkpoints_path_prefix: str,
) -> StreamingQuery:
    """
    Merge in new documents added to the replies bronze table.
    """
    bronze_table = full_table_name(
        bronze_catalog,
        arguments.schemas[arguments.schemas_bronze],
        arguments.tables[arguments.tables_replies],
    )
    streaming = spark.readStream.format("delta").table(bronze_table)
    checkpoint_loc = (
        checkpoints_path_prefix
        + "/docs_noparts/"
        + arguments.tables[arguments.tables_replies]
    )

    return (
        from_bronze_get_replies_df(
            streaming,
            subreddit_idents,
            submission_idents,
            reply_idents,
            bots,
        )
        .writeStream.format("delta")
        .foreachBatch(
            lambda batch_df, batch_id: docs_upsert_replies(
                docs_table, strategy, batch_df
            )
        )
        .option("checkpointLocation", checkpoint_loc)
        .outputMode("append")
        .trigger(availableNow=True)
        .start()
    )


def load_docs_discord(
    spark: SparkSession,
    bronze_catalog: str,
    docs_table: DeltaTable,
    guild_idents: DataFrame,
    channel_idents: DataFrame,
    message_idents: DataFrame,
    strategy: Merge_Strategy,
    checkpoints_path_prefix: str,
) -> StreamingQuery:
    """
    Merge in new documents added to the replies bronze table.
    """
    bronze_table = full_table_name(
        bronze_catalog,
        arguments.schemas[arguments.schemas_bronze],
        arguments.tables[arguments.tables_messages],
    )
    streaming = spark.readStream.format("delta").table(bronze_table)
    checkpoint_loc = (
        checkpoints_path_prefix
        + "/docs_noparts/"
        + arguments.tables[arguments.tables_messages]
    )

    return (
        from_bronze_get_messages_df(
            streaming,
            guild_idents,
            channel_idents,
            message_idents,
        )
        .writeStream.format("delta")
        .foreachBatch(
            lambda batch_df, batch_id: docs_upsert_discord(
                docs_table, strategy, batch_df
            )
        )
        .option("checkpointLocation", checkpoint_loc)
        .outputMode("append")
        .trigger(availableNow=True)
        .start()
    )


def main() -> None:
    """
    Entry point for load docs task
    """
    parser = argparse.ArgumentParser(description=f"Arguments for Trend ETL/{PROGRAM}.")

    parser.add_argument(
        "--bronze-catalog",
        dest="bronze_catalog",
        required=True,
        choices=arguments.catalogs,
        help="The unit catalog to use",
    )

    parser.add_argument(
        "--target-catalog",
        dest="target_catalog",
        required=True,
        choices=arguments.catalogs,
        help="The unit catalog to use",
    )

    parser.add_argument(
        "--checkpoints-path-prefix",
        dest="checkpoints_path_prefix",
        required=True,
        help="The path to the folder (in S3) to store the checkpoints for this table",
    )

    parser.add_argument(
        "--keep-oldest",
        dest="keep_oldest",
        required=True,
        choices=arguments.yesno,
        help="Keep newest or oldest when using the merge to drop duplicates",
    )

    args = parser.parse_args()

    spark = cast(SparkSession, SparkSession.builder.appName(PROGRAM).getOrCreate())

    subreddit_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_subreddit_idents],
    )
    subreddit_idents = spark.read.format("delta").table(subreddit_idents_table)

    submission_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_submission_idents],
    )
    submission_idents = spark.read.format("delta").table(submission_idents_table)

    reply_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_reply_idents],
    )
    reply_idents = spark.read.format("delta").table(reply_idents_table)

    bots_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_reddit_bot_users],
    )
    bots = spark.read.format("delta").table(bots_table)

    guild_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_guild_idents],
    )
    guild_idents = spark.read.format("delta").table(guild_idents_table)

    channel_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_channel_idents],
    )
    channel_idents = spark.read.format("delta").table(channel_idents_table)

    message_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_message_idents],
    )
    message_idents = spark.read.format("delta").table(message_idents_table)

    tablename = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        get_silver_table(args.target_catalog, arguments.tables_docs),
    )

    docs_table = DeltaTable.forName(spark, tablename)
    strategy: Merge_Strategy = (
        "keep-oldest" if args.keep_oldest == "yes" else "keep-newest"
    )

    load_docs_submissions(
        spark,
        args.bronze_catalog,
        docs_table,
        subreddit_idents,
        submission_idents,
        bots,
        strategy,
        args.checkpoints_path_prefix,
    ).awaitTermination()

    load_docs_replies(
        spark,
        args.bronze_catalog,
        docs_table,
        subreddit_idents,
        submission_idents,
        reply_idents,
        bots,
        strategy,
        args.checkpoints_path_prefix,
    ).awaitTermination()

    load_docs_discord(
        spark,
        args.bronze_catalog,
        docs_table,
        guild_idents,
        channel_idents,
        message_idents,
        strategy,
        args.checkpoints_path_prefix,
    ).awaitTermination()


if __name__ == "__main__":
    main()
